/*
 * ProtocolGlobalVariables.h
 *
 * Author: Alirio Boaventura
 * alirioboaventura@gmail.com
 *
 */

// Custom type definitions
typedef unsigned int    Uint32;
typedef unsigned short  Uint16;
typedef unsigned char   Uint8;
typedef int				int16;
typedef char			int8;

#ifndef PROTOCOLGLOBALVARIABLES_H_
#define PROTOCOLGLOBALVARIABLES_H_

// RECEIVER VARIABLES AND CONSTANTS
#define TARGET_BLF 160000 // 160kbps
#define CPU_CLOCK_FREQ 150000000 // 150MHz
#define MAX_TRANSITIONS 1000 // This can be adjusted according to the transponder response
#define MAX_REPLY_LENGTH 200		// maximum length of transponder reply considered here (in bits)
#define SYMB_1	1
#define HALF_SYMB_0 2
#define SYMB_0	3


// TRANSMITTER VARIABLES AND CONSTANTS

// 1-bit DAC
#define N_PW 1
#define N_Tari 2
#define N_0 2
#define N_1 3
#define N_del 4
#define N_RTCal 5
#define N_TRCal 16

#define MAX_COMMAND_LENGTH 200	// maximum length of reader command considered here (in bits)

// Reader command definitions
#define CMD_SELECT 0
#define CMD_QUERY 1
#define CMD_QUERY_REP 2
// .......
#define CMD_ACK_RN16 5
//....

// Modulation type definition
#define MOD_DSB_ASK 1
#define MOD_PR_ASK 2
#define MOD_SSB_ASK 3

// Anti-collision Q parameter definition (Q=0:15 -> number of slots N=1:2^15)
#define Q_0_SLOTS 0
#define Q_2_SLOTS 1
#define Q_4_SLOTS 2
#define Q_8_SLOTS 3
#define Q_16_SLOTS 4
#define Q_32_SLOTS 5
#define Q_64_SLOTS 6
#define Q_128_SLOTS 7
#define Q_256_SLOTS 8
#define Q_512_SLOTS 9
#define Q_1024_SLOTS 10
#define Q_2048_SLOTS 11
#define Q_4096_SLOTS 12
#define Q_8192_SLOTS 13
#define Q_16384_SLOTS 14
#define Q_32768_SLOTS 15


/* Timing (given in microseconds)

 Sequence example: | T_Query_Total(R) | T1 | T_RN16_Total(T) | T2max | T_ACK_RN16_Total(R) | T_EPC_Total(T)| */

// Query
#define Ttari	6.25	// For R->T data rate = 128Kbps
#define	DR	8			// DR (TRcal divide ratio): BLF=DR/TRcal

#define Tdelimiter	12.5

#define Tbit1	(1.5*Ttari)
#define Tbit0	Ttari

#define	TRcal	((float)1000000*DR/TARGET_BLF)
#define RTcal (Tbit0 + Tbit1)

#define Tquery_max	(22*Tbit1) // Worst case scenario happens when all 22 bits are ones
#define T_Query_Total	(Tdelimiter + Ttari + RTcal + TRcal + Tquery_max)

// T1
static const float Tpri = ((float)1000000/TARGET_BLF);	// Transponder data period in microseconds (Tpri=1/BLF)
#define T1	(10*Tpri)
// T2max
#define T2max (20*Tpri)
// T2min
#define T2min	(3*Tpri)

// RN16
#define Tpilot (12*Tpri) // 12-leading zeros
#define Tpreamble	(6*Tpri) // 6 Preamble symbols (1 0 1 0 V 1)
#define Trn16 (16*Tpri)
#define T_RN16_Total (Tpilot + Tpreamble + Trn16)

// ACK RN16
#define Tack_max	(18*Tbit1)  // 18 bits (2 CMD bits + 16 RN16 bits)
#define T_ACK_RN16_Total	(Tdelimiter + Ttari + RTcal + Tack_max)

// EPC
#define Tpc_epc_crc16 (128*Tpri)	// For non-truncated Transponder response: Response = PC + EPC + CRC16 = 128 bits
#define T_EPC_Total 	(Tpilot + Tpreamble + Tpc_epc_crc16)


// Error definition

#define OK 0

#define PILOT_NOT_DETECTED -1

#define PREAMBLE_NOT_DETECTED -2

#define FM0_CODING_ERROR -3

#define CRC16_FAIL -5

#endif /* PROTOCOLGLOBALVARIABLES_H_ */
